@@ -35,7 +35,6 @@ help with disassembling are also handled.
 
 __docformat__ = 'restructuredtext'
 
-import types
 import numpy as np
 import h5py
 
@@ -45,15 +44,18 @@ import os.path as osp
 import mvpa2
 from mvpa2.base import externals
 from mvpa2.base.types import asobjarray
+from mvpa2.misc.support import builtins_mod
 
 if __debug__:
     from mvpa2.base import debug
 
 # don't ask -- makes history re-education a breeze
 universal_classname_remapper = {
-    ('mvpa2.mappers.base', 'FeatureSliceMapper'):
-        ('mvpa2.featsel.base', 'StaticFeatureSelection'),
+    ('mvpa2.mappers.base', 'FeatureSliceMapper'): ('mvpa2.featsel.base',
+                                                   'StaticFeatureSelection'),
 }
+
+
 # Comment: H5Py defines H5Error
 class HDF5ConversionError(Exception):
     """Generic exception to be thrown while doing conversions to/from HDF5
@@ -100,7 +102,7 @@ def hdf2obj(hdf, memo=None):
 
     # if this HDF group has an objref that points to an already recontructed
     # object, simple return this object again
-    if not objref is None and objref in memo:
+    if objref is not None and objref in memo:
         obj = memo[objref]
         if __debug__:
             debug('HDF5', "Use tracked object %s (%i)" % (type(obj), objref))
@@ -127,12 +129,13 @@ def hdf2obj(hdf, memo=None):
     else:
         # check if we have a class instance definition here
         if not ('class' in hdf.attrs or 'recon' in hdf.attrs):
-            raise LookupError("Found hdf group without class instance "
-                    "information (group: %s). Cannot convert it into an "
-                    "object (content: '%s', attributes: '%s')."
-                    % (hdf.name, hdf.keys(), hdf.attrs.keys()))
+            raise LookupError(
+                "Found hdf group without class instance "
+                "information (group: %s). Cannot convert it into an "
+                "object (content: '%s', attributes: '%s')."
+                % (hdf.name, hdf.keys(), hdf.attrs.keys()))
 
-        mod_name = hdf.attrs['module']
+        mod_name = hdf.attrs['module'].decode()
 
         if __debug__:
             if 'class' in hdf.attrs:
@@ -145,9 +148,9 @@ def hdf2obj(hdf, memo=None):
         if 'recon' in hdf.attrs:
             # Custom objects custom reconstructor
             obj = _recon_customobj_customrecon(hdf, memo)
-        elif mod_name != '__builtin__':
+        elif mod_name != builtins_mod:
             # Custom objects default reconstructor
-            cls_name = hdf.attrs['class']
+            cls_name = hdf.attrs['class'].decode()
             if cls_name in ('function', 'type', 'builtin_function_or_method'):
                 # Functions and types
                 obj = _recon_functype(hdf)
@@ -156,7 +159,7 @@ def hdf2obj(hdf, memo=None):
                 obj = _recon_customobj_defaultrecon(hdf, memo)
         else:
             # Built-in objects
-            cls_name = hdf.attrs['class']
+            cls_name = hdf.attrs['class'].decode()
             if __debug__:
                 debug('HDF5', "Reconstructing built-in object '%s'." % cls_name)
             # built in type (there should be only 'list', 'dict' and 'None'
@@ -177,12 +180,13 @@ def hdf2obj(hdf, memo=None):
                 obj = eval(hdf.attrs['name'])
             elif cls_name == 'function':
                 raise RuntimeError("Unhandled reconstruction of built-in "
-                        "function (at '%s')." % hdf.name)
+                                   "function (at '%s')." % hdf.name)
             else:
-                raise RuntimeError("Found hdf group with a builtin type "
-                        "that is not handled by the parser (group: %s). This "
-                        "is a conceptual bug in the parser. Please report."
-                        % hdf.name)
+                raise RuntimeError(
+                    "Found hdf group with a builtin type "
+                    "that is not handled by the parser (group: %s). This "
+                    "is a conceptual bug in the parser. Please report."
+                    % hdf.name)
     #
     # Final post-processing
     #
@@ -200,15 +204,16 @@ def hdf2obj(hdf, memo=None):
 
 def _recon_functype(hdf):
     """Reconstruct a function or type from HDF"""
-    cls_name = hdf.attrs['class']
-    mod_name = hdf.attrs['module']
-    ft_name = hdf.attrs['name']
+    cls_name = hdf.attrs['class'].decode()
+    mod_name = hdf.attrs['module'].decode()
+    ft_name = hdf.attrs['name'].decode()
     if __debug__:
         debug('HDF5', "Load '%s.%s.%s' [%s]"
                       % (mod_name, cls_name, ft_name, hdf.name))
     mod, obj = _import_from_thin_air(mod_name, ft_name, cls_name=cls_name)
     return obj
 
+
 def _get_subclass_entry(cls, clss, exc_msg="", exc=NotImplementedError):
     """In a list of tuples (cls, ...) return the entry for the first
     occurrence of the class of which `cls` is a subclass of.
@@ -219,6 +224,7 @@ def _get_subclass_entry(cls, clss, exc_msg="", exc=NotImplementedError):
             return clstuple
     raise exc(exc_msg % locals())
 
+
 def _update_obj_state_from_hdf(obj, hdf, memo):
     if 'state' in hdf:
         # insert the state of the object
@@ -234,12 +240,13 @@ def _update_obj_state_from_hdf(obj, hdf, memo):
         if __debug__:
             debug('HDF5', "Updated %i state items." % len(state))
 
+
 def _recon_customobj_customrecon(hdf, memo):
     """Reconstruct a custom object from HDF using a custom recontructor"""
     # we found something that has some special idea about how it wants
     # to be reconstructed
-    mod_name = hdf.attrs['module']
-    recon_name = hdf.attrs['recon']
+    mod_name = hdf.attrs['module'].decode()
+    recon_name = hdf.attrs['recon'].decode()
     if __debug__:
         debug('HDF5', "Load from custom reconstructor '%s.%s' [%s]"
                       % (mod_name, recon_name, hdf.name))
@@ -289,9 +296,15 @@ def _recon_customobj_customrecon(hdf, memo):
 
 
 def _import_from_thin_air(mod_name, importee, cls_name=None):
+    # to overcome  TypeError: Item in ``from list'' must be str, not unicode
+    importee = str(importee)
     if cls_name is None:
         cls_name = importee
     try:
+        if mod_name in ('builtins', '__builtin__'):
+            # we might have got it from the hdf saved using another version
+            # of python, so let's map
+            mod_name = builtins_mod
         mod = __import__(mod_name, fromlist=[importee])
     except ImportError as e:
         if mod_name.startswith('mvpa') and not mod_name.startswith('mvpa2'):
@@ -311,8 +324,8 @@ def _import_from_thin_air(mod_name, importee, cls_name=None):
 
 def _recon_customobj_defaultrecon(hdf, memo):
     """Reconstruct a custom object from HDF using the default recontructor"""
-    cls_name = hdf.attrs['class']
-    mod_name = hdf.attrs['module']
+    cls_name = hdf.attrs['class'].decode()
+    mod_name = hdf.attrs['module'].decode()
     if __debug__:
         debug('HDF5', "Load class instance '%s.%s' instance [%s]"
                       % (mod_name, cls_name, hdf.name))
@@ -340,7 +353,7 @@ def _recon_customobj_defaultrecon(hdf, memo):
             getattr(obj, umeth)(cfunc(hdf, memo))
         except NotImplementedError as e:
             if issubclass(cls, tuple) \
-                and hasattr(obj, '_asdict') and hasattr(obj, '_make'):
+                    and hasattr(obj, '_asdict') and hasattr(obj, '_make'):
                 # this is an ugly hack to support NamedTuples -- which
                 # for some fucked-up reason needs this items as args
                 # instead of a sequence like tuple itself
@@ -385,13 +398,15 @@ def _hdf_dict_to_obj(hdf, memo, skip=None):
     else:
         # legacy files had keys as group names
         return dict([(item, hdf2obj(items_container[item], memo=memo))
-                        for item in items_container
-                            if not item in skip])
+                     for item in items_container
+                     if not item in skip])
+
 
 def _hdf_list_to_objarray(hdf, memo):
     if not ('shape' in hdf.attrs):
         if __debug__:
-            debug('HDF5', "Enountered objarray stored without shape (due to a bug "
+            debug(
+                'HDF5', "Enountered objarray stored without shape (due to a bug "
                 "in post 2.1 release).  Some nested structures etc might not be "
                 "loaded incorrectly")
         # yoh: we have possibly a problematic case due to my fix earlier
@@ -420,6 +435,7 @@ def _hdf_list_to_objarray(hdf, memo):
             obj = obj.reshape(shape)
     return obj
 
+
 def _hdf_list_to_obj(hdf, memo, target_container=None):
     """Convert an HDF item sequence into a list
 
@@ -503,7 +519,7 @@ def _hdf_list_to_obj(hdf, memo, target_container=None):
             # we have a value for this item
             items[i] = obj
             # store value for ref if present
-            if not objref is None:
+            if objref is not None:
                 memo[objref] = obj
 
     return items
@@ -530,12 +546,12 @@ def _hdf_to_ndarray(hdf):
             shape = hdf.attrs['shape']
         if 'dtype_names' in hdf.attrs:
             assert('dtype' not in hdf.attrs)
-            names = hdf.attrs['dtype_names']
-            dtypes = hdf.attrs['dtype_types']
+            names = [x for x in hdf.attrs['dtype_names']]
+            dtypes = [x.decode() for x in hdf.attrs['dtype_types']]
             dtype = zip(names, dtypes)
         else:
             assert('dtype' in hdf.attrs)
-            dtype = hdf.attrs['dtype']
+            dtype = hdf.attrs['dtype'].decode()
         obj = np.frombuffer(obj.data, dtype=dtype, count=int(np.prod(shape)))
         obj = obj.reshape(shape, order=['F', 'C'][int(hdf.attrs['c_order'])])
     return obj
@@ -684,9 +700,9 @@ def obj2hdf(hdf, obj, name=None, memo=None, noid=False, **kwargs):
                 dtype = obj_.dtype
                 hdf[name].attrs.create('dtype_names', dtype.names)
                 hdf[name].attrs.create('dtype_types',
-                                       [dtype[i].str for i, _ in enumerate(dtype.names)])
+                                       [dtype[i].str.encode() for i, _ in enumerate(dtype.names)])
             else:
-                hdf[name].attrs.create('dtype', obj_.dtype.str)
+                hdf[name].attrs.create('dtype', obj_.dtype.str.encode())
                 # shape is handled later
 
         if not noid and not is_scalar:
@@ -722,7 +738,7 @@ def obj2hdf(hdf, obj, name=None, memo=None, noid=False, **kwargs):
     #
     # Below handles stuff that cannot be natively stored in HDF5
     #
-    if not name is None:
+    if name is not None:
         if __debug__:
             debug('HDF5', "Store '%s' (ref: %i) in [%s/%s]"
                           % (type(obj), obj_id, hdf.name, name))
@@ -738,7 +754,7 @@ def obj2hdf(hdf, obj, name=None, memo=None, noid=False, **kwargs):
     #
     # Store important flags and references in the group meta data
     #
-    if not noid and not obj is None:
+    if not noid and obj is not None:
         # no refs for basic types
         grp.attrs.create('objref', obj_id)
         # we also note that we processed this object
@@ -765,12 +781,15 @@ def obj2hdf(hdf, obj, name=None, memo=None, noid=False, **kwargs):
         # needs special treatment
         pieces = None
         if __debug__:
-            debug('HDF5', "Failed to reduce '%s' (ref: %i) in [%s]: %s" # (%s)"
-                          % (type(obj), obj_id, hdf.name, te)) #, obj))
+            debug('HDF5',
+                  "Failed to reduce '%s' (ref: %i) in [%s]: %s"  # (%s)"
+                  % (type(obj), obj_id, hdf.name, te))  # , obj))
 
     # common container handling, either __reduce__ was not possible
     # or it was the default implementation
-    if pieces is None or pieces[0].__name__ == '_reconstructor':
+    # _reconstructor could be overloaded by e.g. shogun with _sg_reconstructor
+    # which makes our tender piece of magic fail
+    if pieces is None or pieces[0].__name__.endswith('_reconstructor'):
         # figure out the source module
         if hasattr(obj, '__module__'):
             src_module = obj.__module__
@@ -781,17 +800,16 @@ def obj2hdf(hdf, obj, name=None, memo=None, noid=False, **kwargs):
         # special case: metaclass types NOT instance of a class with metaclass
         if hasattr(obj, '__metaclass__') and hasattr(obj, '__base__'):
             cls_name = 'type'
-
-        if src_module != '__builtin__':
+        if src_module != builtins_mod:
             if hasattr(obj, '__name__'):
-                if not obj.__name__ in dir(__import__(src_module,
+                if obj.__name__ not in dir(__import__(src_module,
                                                       fromlist=[obj.__name__])):
                     raise HDF5ConversionError("Cannot store locally defined "
                                               "function '%s'" % cls_name)
             else:
                 mod_content = dir(__import__(src_module,
                                              fromlist=[cls_name]))
-                if not cls_name in mod_content:
+                if cls_name not in mod_content:
                     # sometimes the class name is not the name of the type
                     # instance imported from the module, e.g. NamedTuple
                     # stored with a different object name
@@ -802,15 +820,15 @@ def obj2hdf(hdf, obj, name=None, memo=None, noid=False, **kwargs):
                     for stuff in mod_content:
                         stuffobj = _import_from_thin_air(src_module, stuff)[1]
                         if isinstance(stuffobj, type) \
-                           and stuffobj.__name__ == cls_name:
+                                and stuffobj.__name__ == cls_name:
                             cls_name = stuff
                             found = True
                     if not found:
                         raise HDF5ConversionError("Cannot store locally defined "
                                                   "class '%s'" % cls_name)
         # store class info (fully-qualified)
-        grp.attrs.create('class', cls_name)
-        grp.attrs.create('module', src_module)
+        grp.attrs.create('class', cls_name.encode())
+        grp.attrs.create('module', src_module.encode())
         if __debug__:
             debug('HDF5', "Stored class info: %s.%s"
                           % (src_module, cls_name))
@@ -821,8 +839,8 @@ def obj2hdf(hdf, obj, name=None, memo=None, noid=False, **kwargs):
             if oname == '<lambda>':
                 raise HDF5ConversionError(
                     "Can't obj2hdf lambda functions. Got %r" % (obj,))
-            grp.attrs.create('name', oname)
-        if isinstance(obj, list) or isinstance(obj, tuple):
+            grp.attrs.create('name', oname.encode())
+        if isinstance(obj, (list, tuple)):
             _seqitems_to_hdf(obj, grp, memo, **kwargs)
         elif isinstance(obj, dict):
             if __debug__:
@@ -836,8 +854,8 @@ def obj2hdf(hdf, obj, name=None, memo=None, noid=False, **kwargs):
         if __debug__:
             debug('HDF5', "Use custom __reduce__ for storage: (%i arguments)."
                           % len(pieces[1]))
-        grp.attrs.create('recon', pieces[0].__name__)
-        grp.attrs.create('module', pieces[0].__module__)
+        grp.attrs.create('recon', pieces[0].__name__.encode())
+        grp.attrs.create('module', pieces[0].__module__.encode())
         if __debug__:
             debug('HDF5', "Stored reconstructor info: %s.%s"
                           % (pieces[0].__module__, pieces[0].__name__))
@@ -846,7 +864,7 @@ def obj2hdf(hdf, obj, name=None, memo=None, noid=False, **kwargs):
         _seqitems_to_hdf(pieces[1], args, memo, **kwargs)
 
     # pull all remaining data from __reduce__
-    if not pieces is None and len(pieces) > 2:
+    if pieces is not None and len(pieces) > 2:
         # there is something in the state
         state = pieces[2]
         if __debug__:
@@ -889,8 +907,8 @@ def h5save(filename, data, name=None, mode='w', mkdir=True, **kwargs):
         if target_dir and not osp.exists(target_dir):
             os.makedirs(target_dir)
     hdf = h5py.File(filename, mode)
-    hdf.attrs.create('__pymvpa_hdf5_version__', '2')
-    hdf.attrs.create('__pymvpa_version__', mvpa2.__version__)
+    hdf.attrs.create('__pymvpa_hdf5_version__', '2'.encode())
+    hdf.attrs.create('__pymvpa_version__', mvpa2.__version__.encode())
     try:
         obj2hdf(hdf, data, name, **kwargs)
     finally:
@@ -917,7 +935,7 @@ def h5load(filename, name=None):
     """
     hdf = h5py.File(filename, 'r')
     try:
-        if not name is None:
+        if name is not None:
             if not name in hdf:
                 raise ValueError("No object of name '%s' in file '%s'."
                                  % (name, filename))
@@ -929,7 +947,7 @@ def h5load(filename, name=None):
             else:
                 # stored objects can only by special groups or datasets
                 if isinstance(hdf, h5py.Dataset) \
-                   or ('class' in hdf.attrs or 'recon' in hdf.attrs):
+                        or ('class' in hdf.attrs or 'recon' in hdf.attrs):
                     # this is an object stored at the toplevel
                     obj = hdf2obj(hdf)
                 else:
